%Hannu Pasanen
%9th of September 2019

%Determines the change in refractive index and 
%extinction coefficient (imaginary part) from transient reflectance and
%transmittance.
%WARNING: If steady-state k is very low (comparable to dk), the fit may bug
%and flip dk form positive to negative or vice-versa
%Fit_delta_n_lin_approx version is recommended for speed
clear;
close all;
%% settings

%Multiplies the weight of the error from transmittance signal. If
%transmittance signal on average is much weaker or stronger than
%reflectance, this can be used to balance the two.
transmittance_multiplier=2;

%Initial estimation for change in refractive index and
%extinction coefficient. This is later changed after each fit to speed up
%the process. You can disable change of initial estimation in the fitting
%section if it causes problems.
delta_n0=[-0.001, 0.001];

%input the index for the delay time you wish to use for fitting
delay_time_index=108;

%% loads measured data (reflectance first)
    [filename, directory_name] = uigetfile('*.dat', 'Select a file');
    fullname = fullfile(directory_name, filename);
    data_dR = load(fullname);  
    
    [filename, directory_name] = uigetfile('*.dat', 'Select a file');
    fullname = fullfile(directory_name, filename);
    data_dT = load(fullname);     
    
%% Fitting
for i=2:length(data_dR(:, delay_time_index))
    
    wavelengths(i-1)=data_dR(i, 1);
    
    disp(wavelengths(i-1)) 
    
    %averages three measured values
    measured_dR=mean([data_dR(i, delay_time_index-1), data_dR(i, delay_time_index), data_dR(i, delay_time_index+1)]);
       
    measured_dT=mean([data_dT(i, delay_time_index-1), data_dT(i, delay_time_index), data_dT(i, delay_time_index+1)]);
    
    %searches for smallest value given by Error_of_simulation as function of delta_n
    y = fminsearch(@(delta_n)Error_of_simulation(wavelengths(i-1), measured_dR, measured_dT, delta_n, transmittance_multiplier),delta_n0);
    
    change(i-1,1)=wavelengths(i-1);    
    change(i-1,2)=y(1);
    change(i-1,3)=y(2);
    
    %changes initial estimation to previously fitted value
    %delta_n0=[y(1), y(2)];

end

%%
figure
hold on
plot(wavelengths,change(:,2))
plot(wavelengths,change(:,3))
xlabel('wavelength (nm)','FontSize',20,'FontName','Arial'); 
ylabel(['\Delta', 'n'],'FontSize',20,'FontName','Arial');
title({['\Delta', 'n at ', num2str(data_dR(1,delay_time_index)),'ps delay']},'Fontsize',20)
legend({['\Delta', 'n'], ['\Delta', 'k']},'Fontsize',20)
hold off

%%
function error=Error_of_simulation(wavelength, measured_dR, measured_dT, delta_n, transmittance_multiplier)

%% simulates the reflectance and transmittance  

[dR_sim, dT_sim]=TRS_function_single_wavelength_uniform_distribution(wavelength,delta_n);

%% gives the error estimate between the simulation and measurements

error=sum((measured_dR*1000-dR_sim).^2)+sum(transmittance_multiplier*(measured_dT*1000-dT_sim).^2);

end

function [dR_sim, dT_sim] = TRS_function_single_wavelength_uniform_distribution(wavelength,delta_n)


[film_thickness, ~, roughness_thickness, roughness_layers, ...
    ~, ~, ...
    ~, angle, ...
    n_air, n_film, n_glass, ...
    ~,~, ...
    coating_thickness, n_coating,...
    substrate_coating_thickness,n_substrate_coating,...
    scf_roughness_thickness,scf_roughness_layers,...
    measurement_side, pol]=simulation_settings(wavelength);

 
%refractive index after excitation
nf_changed=n_film+delta_n(1)+1i*delta_n(2); 

%% thicknesses of the roughness layers

if roughness_layers==0
    roughness_1=[];
else
    z = linspace(0,1,roughness_layers)*roughness_thickness;
    dz = roughness_thickness/(roughness_layers); %thickness of one roughness layer
    roughness_1=ones(1,roughness_layers)*dz;    
end

if scf_roughness_layers==0
    roughness_2=[];
else
z_2 = linspace(0,1,scf_roughness_layers)*scf_roughness_thickness;
dz_2=scf_roughness_thickness/scf_roughness_layers; %thickness of one roughness layer
roughness_2=ones(1,scf_roughness_layers)*dz_2;
end

%thicknesses of all the layers
layers_thicknesses_total=[coating_thickness roughness_1 film_thickness roughness_2 substrate_coating_thickness];

%% refractive indexes

%% roughness layer 1
if roughness_layers==0
    nf_roughness=[];
    nf_roughness_changed=[];
else    
        V=1/3 / roughness_thickness^2 * (z.^3 - (z-dz).^3); %volume of each roughness slice
        V_frac_1=V/dz;                                  %volume fraction of each roughness slice
        nf_roughness = sqrt(V_frac_1.*n_film.^2+(1-V_frac_1).*n_coating(end).^2);  %refractive index of roughness layer in steady state
        nf_roughness_changed = sqrt(V_frac_1.*nf_changed.^2+(1-V_frac_1).*n_coating(end).^2);
end

%% roughness layer 2
if scf_roughness_layers==0
    nf_roughness_2=[];
    nf_roughness_2_changed=[];
else    
    V_2 =fliplr( 1/3 / scf_roughness_thickness^2 * (z_2.^3 - (z_2-dz_2).^3)); %volume of each roughness slice    
    V_frac_2 = V_2/dz_2;    
    nf_roughness_2 = sqrt( V_frac_2  .*n_film.^2 + (1- V_frac_2 ).*n_substrate_coating(1).^2); %refractive index of roughness layer in steady state
    nf_roughness_2_changed = sqrt( V_frac_2  .*nf_changed.^2 + (1- V_frac_2 ).*n_substrate_coating(1).^2);    
end

%%

% combines roughness and bulk layers
layers_n_original=[n_coating...
    nf_roughness...
    n_film ...
    nf_roughness_2...
    n_substrate_coating];

layers_n_changed=[n_coating...
    nf_roughness_changed...
    nf_changed ...
    nf_roughness_2_changed...
    n_substrate_coating];
%%

polarization='TM';

    % Steady-state 
       
    [R0_with_substrate,T0_with_substrate]=transfer_matrix(n_air,n_glass,layers_n_original,layers_thicknesses_total,wavelength,angle,polarization,measurement_side);
        
    
    % after excitation 

    [R_sim_with_substrate,T_sim_with_substrate]=transfer_matrix(n_air,n_glass,layers_n_changed,layers_thicknesses_total,wavelength,angle,polarization,measurement_side);
          
     dR_sim_TM=R_sim_with_substrate/R0_with_substrate*100 -100;
     dT_sim_TM=T_sim_with_substrate/T0_with_substrate*100 -100;
        
    

%%
polarization='TE';

    % Steady-state 
    
    [R0_with_substrate,T0_with_substrate]=transfer_matrix(n_air,n_glass,layers_n_original,layers_thicknesses_total,wavelength,angle,polarization,measurement_side);
        
    
    % after excitation 

    [R_sim_with_substrate,T_sim_with_substrate]=transfer_matrix(n_air,n_glass,layers_n_changed,layers_thicknesses_total,wavelength,angle,polarization,measurement_side);
          
     dR_sim_TE=R_sim_with_substrate/R0_with_substrate*100 -100;
     dT_sim_TE=T_sim_with_substrate/T0_with_substrate*100 -100;

%% average of TE and TM, change to mOD units
dR_sim=-log10((pol*dR_sim_TE+(1-pol)*dR_sim_TM)/100+1)*1000;
dT_sim=-log10((pol*dT_sim_TE+(1-pol)*dT_sim_TM)/100+1)*1000;

end