function [R_total,T_total]=transfer_matrix(n1,n_glass,n_film,thickness,lambda,theta,polarization, measurement_side)

n_film=real(n_film)-1j*imag(n_film); %changes n+ik to n-ik
n_glass=real(n_glass)-1j*imag(n_glass);

% substrate absorption

Substrate_absorbance=4*pi*imag(n_glass)*10^6/lambda;

%angle inside the substrate

n2=real(n_glass);

theta_2=asind(n1./n2 *sind(theta));

%increase of substrate absorption due to angle

Substrate_absorbance=Substrate_absorbance/cosd(theta_2);

%reflectance from air-substrate interface  

if strcmp(polarization,'TE')
    
    R_air_glass=abs((n1*cosd(theta)-n2*cosd(theta_2)) ...
    /(n1*cosd(theta)+n2*cosd(theta_2) )).^2;
    
elseif strcmp(polarization,'TM')
    
    R_air_glass=abs((-n2*cosd(theta)+n1*cosd(theta_2)) ...
    /(n2*cosd(theta)+n1*cosd(theta_2) )).^2;
    
else
    error('Polarization must be TE or TM!')
end 

%reflectance from substrate-air interface

R_glass_air= R_air_glass;

%%
if strcmp(measurement_side,'top')
%returns reflectance and transmittance of thin film on substrate

    %reflectance/transmittance of air-film-substrate
    [~,~,R_air_film,T_air_film]=tmatrix_filmstack_2018(n1,n_glass,n_film,thickness,lambda,theta,polarization);  
   
    %reflectance/transmittance of substrate-film-air 
    [~,~,R_glass_film,T_glass_film]=tmatrix_filmstack_2018(n_glass,n1,fliplr(n_film),fliplr(thickness),lambda,theta,polarization);    

    R_total=R_air_film+...
        T_air_film*R_glass_air*T_glass_film*(exp(2*Substrate_absorbance))+...
        T_air_film*R_glass_air^2*T_glass_film*(exp(4*Substrate_absorbance))*R_glass_film;
            
    T_total=T_air_film*exp(Substrate_absorbance)*(1-R_glass_air)+...
        T_air_film*exp(3*Substrate_absorbance)*R_glass_air*R_glass_film*(1-R_glass_air)+...
        T_air_film*exp(5*Substrate_absorbance)*R_glass_air^2*R_glass_film^2*(1-R_glass_air);
    
elseif strcmp(measurement_side,'bottom') 
    
    %reflectance/transmittance of substrate-film-air 
    [~,~,R_glass_film,T_glass_film]=tmatrix_filmstack_2018(n_glass,n1,fliplr(n_film),fliplr(thickness),lambda,theta,polarization);
    
    R_total=R_air_glass+...
            exp(2*Substrate_absorbance) *R_glass_film *(1-R_glass_air)*(1-R_air_glass) +...
            exp(4*Substrate_absorbance) *R_glass_film^2 *(1-R_glass_air)*R_glass_air*(1-R_air_glass);
        
    T_total=(1-R_air_glass) *exp(Substrate_absorbance) *T_glass_film+...
            (1-R_air_glass) *exp(3*Substrate_absorbance) *R_glass_air *R_glass_film*T_glass_film...
            ;
    
else
    error('Measurement side has to be top or bottom')
end

%%
function [r,t,R,T]=tmatrix_filmstack_2018(n1,n2,n,L,lambda,theta,polarization)

%[R,T]=tmatrix_filmstack(n1,n2,n,L,lambda,theta,polarization)

% Calculates the reflectance R and transmittance T or a thin film stack
% using the transfer matrix method.

% n1 = refractive index of incidence side material
% n2 = refractive index after the thin film stack
% n = refractive indices of the layers of the stack, vector
% L = thicknesses of the layers of the stack, vector [in nanometers]
% lambda = wavelength [in nanometers]
% theta = incidence angle [in degrees]
% polarization = 'TE' or 'TM'

% Tapio Niemi and Janne Simonen, 
% FYS-6606 Photonics - course
% Tampere University of Technology, 2018
% Tapio.K.Niemi@tut.fi


% Check validity of input parameters

if length(n)~=length(L)
    error('n and L must be of the same length!')
end

% refractive index vector including surrounding materials
n=[n1 n n2];


% Number of refractive indices = length of vector n
% (so we have N materials, N-1 interfaces and N-2 layers)
N=length(n);



% Tangential  component of the incident wavevector k.
% Important: this is the same in each of the layers!
% Remember that sin() wants radians but theta is in degrees
kt=2*pi/lambda*n1*sin(theta*pi/180);

% Normal components of wave vectors in all layers from Pythagoras
% Notice, this is a vector of length N
kn=sqrt((2*pi/lambda*n).^2-kt^2);

% Ensure exponential decay in exit material
% This is meaningful if total internal reflection takes place
kn=real(kn)-1j*abs(imag(kn));

% initialize the 2x2 transfer matrix
matrix=eye(2);

% This part depends on the polarization
% Reflectance and transmittance coefficients of each interface,
% vectors of length N-1

if strcmp(polarization,'TE')

    r1=(kn(1:N-1)-kn(2:N))./(kn(1:N-1)+kn(2:N));
    t1=2*kn(1:N-1)./(kn(1:N-1)+kn(2:N));
    
elseif strcmp(polarization,'TM')
    
    r1=(n(1:N-1).^2.*kn(2:N)-n(2:N).^2.*kn(1:N-1))./(n(2:N).^2.*kn(1:N-1)+n(1:N-1).^2.*kn(2:N));
    t1=(2*n(2:N).*n(1:N-1).*kn(1:N-1))./(n(2:N).^2.*kn(1:N-1)+n(1:N-1).^2.*kn(2:N));  
    
else
    error('Polarization must be TE or TM!')
end

% Generate the T and P matrices for each layer/interface
% Let's loop through all the layers, its fast enough

for M=1:N-2
    T1=1/t1(M)*[1 r1(M);r1(M) 1];
    P=[exp(1i*kn(M+1)*L(M)) 0 ; 0 exp(-1i*kn(M+1)*L(M))];
    
    % multiply the total transfer matrix by P and T, repeat if needed
    matrix=matrix*T1*P;
end

% We have almost the full system matrix but are missing the final
% interface T

matrix=matrix*(1/t1(N-1)*[1 r1(N-1);r1(N-1) 1]); 

% Results:
% reflectance and transmittance of optical power
% can be solved from the elements of the total tranfer matrix
% we need theta2=propagation angle in output 

theta2=atan(kt/real(kn(end)));

r=(matrix(2,1)/matrix(1,1));
t=1./matrix(1,1);

R=abs(r).^2;

% Transmittance by two methods
% T=real(n2)*cos(theta2)/(n1*cos(theta/180*pi))*abs(t).^2;

% If you use T=1-R you can not analyse absorption in the structure
% T+R+A=1
% For lossless materials its OK.

T=real(kn(end)./kn(1))*abs(t).^2;

end
end